import { render, screen, waitFor } from '@testing-library/react';
import React from 'react';
import '@testing-library/jest-dom';
import userEvent from '@testing-library/user-event';
import { CreateRecordedQuery } from './RecordedQueryCreateModal';
import { DataQuery, DataSourceInstanceSettings } from '@grafana/data';
import { config } from '@grafana/runtime';
import { of } from 'rxjs';
import { selectOptionInTest } from '@grafana/ui';

const fetchMock = jest.fn().mockReturnValue(of({ data: { message: 'hello' } }));
const settingsMock = jest.fn().mockReturnValue({ id: 36 });

jest.mock('@grafana/runtime', () => {
  const original = jest.requireActual('@grafana/runtime');
  const mockedRuntime = { ...original };

  mockedRuntime.getDataSourceSrv = () => {
    return {
      getInstanceSettings: settingsMock,
    };
  };

  mockedRuntime.getBackendSrv = () => ({
    fetch: fetchMock,
  });

  return mockedRuntime;
});

describe('RecordedQueryCreateModal', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });

  describe('does not render the modal', () => {
    it('without license ', () => {
      config.licenseInfo.hasLicense = false;
      config.featureToggles.recordedQueries = true;
      render(<CreateRecordedQuery onChangeDataSource={() => {}} />);

      expect(screen.queryByTitle('Create Recorded Query')).not.toBeInTheDocument();
    });

    it('without feature toggle', () => {
      config.licenseInfo.hasLicense = true;
      config.featureToggles.recordedQueries = false;
      render(<CreateRecordedQuery onChangeDataSource={() => {}} />);

      expect(screen.queryByTitle('Create Recorded Query')).not.toBeInTheDocument();
    });
  });

  describe('with license and feature toggle', () => {
    beforeEach(() => {
      config.licenseInfo.hasLicense = true;
      config.featureToggles.recordedQueries = true;
    });

    it('validates the form for required values', async () => {
      render(<CreateRecordedQuery onChangeDataSource={() => {}} />);

      userEvent.click(screen.getByTitle('Create Recorded Query'));
      userEvent.click(screen.getByText('Start recording query'));

      await waitFor(() => {
        expect(screen.getByText('Name is required')).toBeInTheDocument();
      });
    });

    it('posts a recording rule with existing queries to the backend endpoint', async () => {
      const query: DataQuery = { refId: 'A' };
      const queries: Array<Partial<DataQuery>> = [{ refId: 'A' }];
      const dataSource = { name: 'ds-name' } as DataSourceInstanceSettings;

      render(
        <CreateRecordedQuery query={query} queries={queries} dataSource={dataSource} onChangeDataSource={() => {}} />
      );
      await fillRecordedQueryForm();

      userEvent.click(screen.getByText('Start recording query'));

      await waitFor(() => {
        expect(fetchMock).toHaveBeenCalledWith({ ...rrQuery, url: 'api/recording-rules', showErrorAlert: false });
      });
    });

    it('posts a recording rule to the test backend endpoint', async () => {
      const query: DataQuery = { refId: 'A' };
      const queries: Array<Partial<DataQuery>> = [{ refId: 'A' }];
      const dataSource = { name: 'ds-name' } as DataSourceInstanceSettings;

      render(
        <CreateRecordedQuery query={query} queries={queries} dataSource={dataSource} onChangeDataSource={() => {}} />
      );

      await fillRecordedQueryForm();

      userEvent.click(screen.getByLabelText('Test the recorded query'));

      await waitFor(() => {
        expect(fetchMock).toHaveBeenCalledWith({
          ...rrQuery,
          url: 'api/recording-rules/test',
          showSuccessAlert: false,
          showErrorAlert: false,
        });
      });
    });

    it('uses the data source name from the query if it exists', async () => {
      const query: DataQuery = { refId: 'A' };
      const queries: Array<Partial<DataQuery>> = [{ refId: 'A', datasource: 'query-datasource' }];
      const dataSource = { name: 'ds-name' } as DataSourceInstanceSettings;

      render(
        <CreateRecordedQuery query={query} queries={queries} dataSource={dataSource} onChangeDataSource={() => {}} />
      );
      await fillRecordedQueryForm();

      userEvent.click(screen.getByText('Start recording query'));

      await waitFor(() => {
        expect(settingsMock).toHaveBeenCalledWith('query-datasource');
      });
    });
  });
});

async function fillRecordedQueryForm() {
  userEvent.click(screen.getByTitle('Create Recorded Query'));

  const intervalSelect = screen.getByLabelText('Interval');
  const rangeSelect = screen.getByLabelText('Range');

  userEvent.type(screen.getByLabelText('Add recorded query name'), 'q-name');
  userEvent.type(screen.getByLabelText('Add recorded query description'), 'desc');
  await selectOptionInTest(intervalSelect, '30m');
  await selectOptionInTest(rangeSelect, '12h');
  userEvent.click(screen.getByLabelText('Record the count of the returned value'));
}

const rrQuery = {
  data: {
    active: true,
    count: true,
    description: 'desc',
    interval: 1800,
    name: 'q-name',
    queries: [
      {
        datasourceId: 36,
        refId: 'A',
      },
    ],
    range: 43200,
    target_ref_id: 'A',
  },
  method: 'POST',
};
