import React, { useState } from 'react';
import { css } from '@emotion/css';
import { config, getDataSourceSrv } from '@grafana/runtime';
import {
  Alert,
  AlertVariant,
  Button,
  Field,
  Form,
  Icon,
  InlineFieldRow,
  Input,
  Modal,
  Select,
  Switch,
  useStyles2,
} from '@grafana/ui';
import { AppEvents, DataQuery, DataSourceInstanceSettings, GrafanaTheme2, SelectableValue } from '@grafana/data';
import { QueryOperationAction } from 'app/core/components/QueryOperationRow/QueryOperationAction';
import { IdDataQuery, RecordedQuery } from '../types/recordedQuery';
import { saveRecordedQuery, testRecordedQuery } from './state/actions';
import appEvents from 'app/core/app_events';
import { useUniqueId } from 'app/plugins/datasource/influxdb/components/useUniqueId';

interface Props {
  query?: DataQuery;
  queries?: Array<Partial<DataQuery>>;
  onAddQuery?: (query: DataQuery) => void;
  onChangeDataSource?: (ds: DataSourceInstanceSettings) => void;
  dataSource?: DataSourceInstanceSettings;
}

interface SaveAddRecordedQuery {
  name: string;
  description: string;
  aggregationType: string;
  interval: string;
  range: string;
  isCount: boolean;
}

const timeIntervals: Array<SelectableValue<number>> = [
  { label: '30s', value: 30 },
  { label: '1m', value: 60 },
  { label: '5m', value: 300 },
  { label: '30m', value: 1800 },
  { label: '1h', value: 3600 },
  { label: '12h', value: 43200 },
  { label: '1d', value: 86400 },
  { label: '1w', value: 604800 },
  { label: '1M', value: 2592000 },
];

export const CreateRecordedQuery = ({ query, queries, dataSource }: Props) => {
  const [isOpen, setIsOpen] = useState(false);
  const [alertSeverity, setAlertSeverity] = useState<AlertVariant>('info');
  const [alertMessage, setAlertMessage] = useState('');
  const [intervalSeconds, setIntervalSeconds] = useState<number>(timeIntervals[0].value!);
  const [rangeSeconds, setRangeSeconds] = useState<number>(timeIntervals[0].value!);
  const intervalId = useUniqueId();
  const rangeId = useUniqueId();
  const styles = useStyles2(getStyles);

  const toRecordedQuery = (data: SaveAddRecordedQuery): RecordedQuery => {
    return {
      target_ref_id: query ? query.refId : '',
      name: data.name,
      description: data.description,
      interval: intervalSeconds,
      range: rangeSeconds,
      count: data.isCount,
      active: true,
      queries: queries?.map((q) => {
        const idQ = q as IdDataQuery;
        const dataSourceName = q.datasource || dataSource?.name;
        const ds = getDataSourceSrv().getInstanceSettings(dataSourceName);
        idQ.datasourceId = ds ? ds.id : -1;
        return idQ;
      }),
    } as RecordedQuery;
  };

  const showRecordQuery = config?.licenseInfo?.hasLicense && config?.featureToggles?.recordedQueries;
  if (!showRecordQuery) {
    return null;
  }

  const closeModal = () => {
    setAlertMessage('');
    setAlertSeverity('info');
    setIsOpen(false);
  };

  return (
    <>
      <QueryOperationAction
        title="Create Recorded Query"
        icon="history"
        onClick={() => {
          setIsOpen(true);
        }}
      />
      <Modal
        isOpen={isOpen}
        title={'Create Record Query'}
        onDismiss={closeModal}
        className={css`
          width: 500px;
        `}
      >
        <Form
          defaultValues={{
            name: '',
            description: '',
            aggregationType: '',
            interval: '',
            range: '',
            isCount: false,
          }}
          onSubmit={async (data) => {
            saveRecordedQuery(toRecordedQuery(data))
              .then(() => {
                closeModal();
                appEvents.emit(AppEvents.alertSuccess, [`Created Recorded Query: ${data.name}`]);
              })
              .catch((error) => {
                appEvents.emit(AppEvents.alertError, [error.data.message]);
              });
          }}
          validateOn="all"
        >
          {({ register, errors, getValues }) => (
            <>
              <Field label="Name" invalid={!!errors.name} error={errors.name?.message}>
                <Input
                  {...register('name', { required: { value: true, message: 'Name is required' } })}
                  aria-label="Add recorded query name"
                  autoFocus
                />
              </Field>
              <Field label="Description">
                <Input {...register('description')} aria-label="Add recorded query description" />
              </Field>
              <InlineFieldRow>
                <div className={styles.select}>
                  <Field label="Interval" htmlFor={intervalId}>
                    <Select
                      inputId={intervalId}
                      options={timeIntervals}
                      onChange={(v: SelectableValue<number>) => {
                        setIntervalSeconds(v.value!);
                      }}
                      value={intervalSeconds}
                      width={20}
                    />
                  </Field>
                </div>
                <div className={styles.select}>
                  <Field label="Range" htmlFor={rangeId}>
                    <Select
                      inputId={rangeId}
                      options={timeIntervals}
                      onChange={(v: SelectableValue<number>) => {
                        setRangeSeconds(v.value!);
                      }}
                      value={rangeSeconds}
                      width={20}
                    />
                  </Field>
                </div>
              </InlineFieldRow>
              <Field label="Count Query Results">
                <Switch {...register('isCount')} aria-label="Record the count of the returned value" />
              </Field>
              <Alert
                severity={alertSeverity}
                title={'Test Recorded Query'}
                buttonContent={
                  <div aria-label="Test the recorded query">
                    <Icon name="sync" />
                  </div>
                }
                onRemove={() => {
                  testRecordedQuery(toRecordedQuery(getValues()))
                    .then((value) => {
                      setAlertSeverity('info');
                      setAlertMessage(`Result: ${value.data.message}`);
                    })
                    .catch((error) => {
                      setAlertSeverity('error');
                      setAlertMessage(error.data.message);
                    });
                }}
              >
                <>{alertMessage}</>
              </Alert>
              <Modal.ButtonRow>
                <Button
                  type="button"
                  variant="secondary"
                  onClick={closeModal}
                  fill="outline"
                  aria-label="Cancel recording query button"
                >
                  Cancel
                </Button>
                <Button type="submit" aria-label="Start recording query button">
                  Start recording query
                </Button>
              </Modal.ButtonRow>
            </>
          )}
        </Form>
      </Modal>
    </>
  );
};

const getStyles = (theme: GrafanaTheme2) => {
  return {
    select: css`
      margin-right: ${theme.spacing(1)};
    `,
  };
};
